/*
 * IBM Confidential
 * OCO Source Materials
 * 5724-U98, 5655-V48, 5724-U50, 5655-V46
 *  Copyright IBM Corp. 2013
 * The source code for this program is not published or other -
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
header {
	package com.ibm.nex.parser.oim.distributed.filename;
}


{ @SuppressWarnings("all") }
class DistributedFilenameParser extends Parser;

options {
	buildAST = true;
	k = 4;
	defaultErrorHandler=false;
}

filename
	: filenameSegment | ((filenameSegment)? macroBlock (filenameSegment)?)
	;

// File name segment - a segment that can contain any character except an open or close macro symbol.
filenameSegment
	: (~(OPENMACROBLOCK | CLOSEMACROBLOCK))+
	;

macroBlock
	: OPENMACROBLOCK (WHITESPACE)? macroSegment (separator macroSegment)* (separator)? CLOSEMACROBLOCK
	;

macroSegment
	: (macro | LITERAL)
	;

/**
	A separator is either plain whitespace, or a comma surrounded by whitespace.
	Originally, this rule was written as:
	
	(WHITESPACE)? (COMMA)? (WHITESPACE)?
	
	Which caused problems because the rule could resolve to an empty set of tokens. The end result would be that the separator validation was meaningless. 	
*/ 
separator
	: WHITESPACE | ((WHITESPACE)? COMMA (WHITESPACE)?)
	;

macro
	: MACRO_DAY |
	  MACRO_DD |
	  MACRO_DE |
	  MACRO_DW |
	  MACRO_D |
	  MACRO_DY |
	  MACRO_DDD |
	  MACRO_HH |
	  MACRO_HHMM |
	  MACRO_MIN |
	  MACRO_MM |
	  MACRO_MON |
	  MACRO_OBJNAME |
	  MACRO_SEQ |
	  MACRO_SS |
	  MACRO_USER |
	  MACRO_WE |
	  MACRO_WW |
	  MACRO_YY
	;

{ @SuppressWarnings("all") }
class DistributedFilenameLexer extends Lexer;

options {
	k = 5;
	charVocabulary = '\u0000'..'\uFFFE';
}

LITERAL
	: 
	  LITERALHEADERCHAR (LITERALBODYCHAR)*
	;

// Protected lexer rules are helper rules available only to the lexer, not the parser. The lexer will never return protected tokens.
protected
LITERALHEADERCHAR
	:  	 
	~(
		'<' | 
		'>' | 
		'$' |
		',' |
		' ' | '\t' | '\u001a'
	  )	
	;

protected
LITERALBODYCHAR
	: LITERALHEADERCHAR | MACROPREFIX
	;

OPENMACROBLOCK
	: '<'
	;

CLOSEMACROBLOCK
	: '>' 
	;

COMMA
	: ','
	;

MACRO_DAY
	: MACROPREFIX D A Y
	;

MACRO_DD
	: MACROPREFIX D D
	;

MACRO_DE
	: MACROPREFIX D E
	;

MACRO_DW
	: MACROPREFIX D W
	;

MACRO_D
	: MACROPREFIX D
	;

MACRO_DY
	: MACROPREFIX D Y
	;

MACRO_DDD
	: MACROPREFIX D D D
	;

MACRO_HH
	: MACROPREFIX H H
	;

MACRO_HHMM
	: MACROPREFIX H H M M
	;

MACRO_MIN
	: MACROPREFIX M I N
	;

MACRO_MM
	: MACROPREFIX M M
	;

MACRO_MON
	: MACROPREFIX M O N
	;

MACRO_OBJNAME
	: MACROPREFIX O B J N A M E
	;

MACRO_SEQ
	: MACROPREFIX S E Q (INTEGER)?
	;

MACRO_SS
	: MACROPREFIX S S
	;

MACRO_USER
	: MACROPREFIX U S E R
	;

MACRO_WE
	: MACROPREFIX W E
	;

MACRO_WW
	: MACROPREFIX W W
	;

MACRO_YY
	: MACROPREFIX Y Y
	;

/**
	NOTE: MACROPREFIX is not protected because it may appear in part of a filename segment. 
	A filename segment is made of any token that is not a macro block token (see filenameSegment above).
 	In this example:

		"C:\\IBM\\InfoSphere\\Optim Distributed\\data\\$dollar123integer,comma,$123,$123mixed.ECM"

	The tokens are:

		LITERAL:	"C:\\IBM\\InfoSphere\\Optim Distributed\\data\\$dollar123integer"
		COMMA:		","
		LITERAL:	"comma"
		COMMA:		","
		MACROPREFIX:"$"
		LITERAL:	"123"
		COMMA:		","
		MACROPREFIX:"$"
		LITERAL:	"123mixed.ECM"
		
	If the MACROPREFIX token is protected, then the character is not recognized as a valid token within the lexer.
*/		 
MACROPREFIX
	: '$'
	;

WHITESPACE
	: (WS)+
	;

// Normally, whitespace characters are skipped. However, the filename segments may include whitespace, so we don't want to remove them.
protected WS
	: (' ' | '\t' | '\u001a')
	;

// Protected lexer rules are helper rules available only to the lexer, not the parser. The lexer will never return protected tokens.
protected INTEGER
	: (DIGIT)+
	;

protected DIGIT
	: '0'..'9'
	;

protected A:('a'|'A');
protected B:('b'|'B');
protected C:('c'|'C');
protected D:('d'|'D');
protected E:('e'|'E');
protected F:('f'|'F');
protected G:('g'|'G');
protected H:('h'|'H');
protected I:('i'|'I');
protected J:('j'|'J');
protected K:('k'|'K');
protected L:('l'|'L');
protected M:('m'|'M');
protected N:('n'|'N');
protected O:('o'|'O');
protected P:('p'|'P');
protected Q:('q'|'Q');
protected R:('r'|'R');
protected S:('s'|'S');
protected T:('t'|'T');
protected U:('u'|'U');
protected V:('v'|'V');
protected W:('w'|'W');
protected X:('x'|'X');
protected Y:('y'|'Y');
protected Z:('z'|'Z');